require "src/vector"
require "src/triangle"
require "src/mesh"
require "src/matrix"

function love.load()
    ---------------------------setup screen size-----------------------------------------
    screen = {}
    screen.dpiScale = 2--love.graphics.getDPIScale()
    screen.width, screen.height = love.window.getDesktopDimensions()
    screen.width, screen.height = screen.width / screen.dpiScale, screen.height / screen.dpiScale
    love.window.setMode(screen.width, screen.height)
    love.window.setTitle("3D Renderer")
    
    ----------------------------Cube Mesh-------------------------------------------------
    local topLeftF = Vector.new(0,1,0)
    local topRightF = Vector.new(1,1,0)
    local bottomRightF = Vector.new(1,0,0)
    local bottomLeftF =  Vector.new(0,0,0)
    local topLeftB = Vector.new(0,1,1)
    local topRightB = Vector.new(1,1,1)
    local bottomRightB = Vector.new(1,0,1)
    local bottomLeftB =  Vector.new(0,0,1)

    local triangles = {}
    local t = Triangle.new({topLeftF, topRightF, bottomRightF})
    table.insert(triangles, t)
    local t = Triangle.new({topLeftF, bottomLeftF, bottomRightF})
    table.insert(triangles, t)
    local t = Triangle.new({topRightF, topRightB, bottomRightB})
    table.insert(triangles, t)
    local t = Triangle.new({topRightF, bottomRightB, bottomRightB})
    table.insert(triangles, t)
    local t = Triangle.new({topRightB, topLeftB, bottomLeftB})
    table.insert(triangles, t)
    local t = Triangle.new({topRightB, bottomRightB, bottomLeftB})
    table.insert(triangles, t)
    local t = Triangle.new({topLeftB, topLeftF, bottomLeftF})
    table.insert(triangles, t)
    local t = Triangle.new({topLeftB, bottomLeftB, bottomLeftF})
    table.insert(triangles, t)
    local t = Triangle.new({topLeftB, topRightB, topRightF})
    table.insert(triangles, t)
    local t = Triangle.new({topLeftB, topLeftF, topRightF})
    table.insert(triangles, t)
    local t = Triangle.new({bottomRightB, bottomRightF, bottomLeftF})
    table.insert(triangles, t)
    local t = Triangle.new({bottomRightB, bottomLeftB, bottomLeftF})
    table.insert(triangles, t)

    cube = Mesh.new(triangles)
    
    a = {"a"}
    b = {}
    table.insert(b, a)
    c = {} 
    table.insert(c, b)

    ----------------------------Projection Matrix---------------------------------------------
    local near = 0.1
    local far = 1000
    local fov = 90 
    local aspectRatio = screen.height / screen.width
    local fovRad = 1 / math.tan(fov*0.5/180* math.pi)
    
    matProj = Matrix.new(4, 4)


    matProj.m[1][1] = aspectRatio * fovRad
    matProj.m[2][2] = fovRad
    matProj.m[3][3] = far / (far - near)
    matProj.m[4][3] = (-far * near) / (far - near)
    matProj.m[3][4] = 1
    matProj.m[4][4] = 0
end

function love.update(dt)

end

function love.draw()
    for i, tris in ipairs(cube.triangles) do
        local triProjected = Triangle.new({Vector.new(0,0,0), Vector.new(0,0,0), Vector.new(0,0,0)} )
        local triTranslated = Triangle.new({Vector.new(0,0,0), Vector.new(0,0,0), Vector.new(0,0,0)} )
        triTranslated = tris
        
        triTranslated.vectors[1].z = tris.vectors[1].z +3
        triTranslated.vectors[2].z = tris.vectors[2].z +3
        triTranslated.vectors[3].z = tris.vectors[3].z +3

        triProjected.vectors[1] = MultiplyMatrixVector(triTranslated.vectors[1], matProj);
        triProjected.vectors[2] = MultiplyMatrixVector(triTranslated.vectors[2], matProj);
        triProjected.vectors[3] = MultiplyMatrixVector(triTranslated.vectors[3], matProj);


        --scale into view
        triProjected.vectors[1].x = triProjected.vectors[1].x+1;  triProjected.vectors[1].y = triProjected.vectors[1].y+1;
        triProjected.vectors[2].x = triProjected.vectors[2].x+1;  triProjected.vectors[2].y = triProjected.vectors[2].y+1;
        triProjected.vectors[3].x = triProjected.vectors[3].x+1;  triProjected.vectors[3].y = triProjected.vectors[3].y+1;

        triProjected.vectors[1].x = triProjected.vectors[1].x * 0.5 * screen.width;
        triProjected.vectors[1].y = triProjected.vectors[1].y * 0.5 * screen.height;
        triProjected.vectors[2].x = triProjected.vectors[2].x * 0.5 * screen.width;
        triProjected.vectors[2].y = triProjected.vectors[2].y * 0.5 * screen.height;
        triProjected.vectors[3].x = triProjected.vectors[3].x * 0.5 * screen.width;
        triProjected.vectors[3].y = triProjected.vectors[3].y * 0.5 * screen.height;

        love.graphics.polygon("line", 
        triProjected.vectors[1].x, triProjected.vectors[1].y,
        triProjected.vectors[2].x, triProjected.vectors[2].y, 
        triProjected.vectors[3].x, triProjected.vectors[3].y)
    end
end

function MultiplyMatrixVector(i,matrix)
    local o = {x=0, y=0, z=0}
    o.x = i.x * matrix.m[1][1] + i.y * matrix.m[2][1] + i.z * matrix.m[3][1] + matrix.m[4][1]
    o.y = i.x * matrix.m[1][2] + i.y * matrix.m[2][2] + i.z * matrix.m[3][2] + matrix.m[4][2]
    o.z = i.x * matrix.m[1][3] + i.y * matrix.m[2][3] + i.z * matrix.m[3][3] + matrix.m[4][3]
    local w = i.x * matrix.m[1][4] + i.y * matrix.m[2][4] + i.z * matrix.m[3][4] + matrix.m[4][4]
    if not  w == 0 then
        o.x = o.x / w 
        o.y = o.y / w 
        o.z = o.z / w
    end
    return o
end